use bevy::prelude::*;
use std::time::Duration;

use crate::components::{Enemy, EnemyBullet, EnemyState, EnemyType, FormationTarget, TractorBeam, Captured};
use crate::constants::{ // Only keeping used constants
    ENEMY_BULLET_SIZE, ENEMY_SIZE, ENEMY_SPEED, WINDOW_HEIGHT, WINDOW_WIDTH,
    TRACTOR_BEAM_WIDTH, TRACTOR_BEAM_DURATION, TRACTOR_BEAM_COLOR, CAPTURE_DURATION,
};
use crate::resources::{
    AttackDiveTimer, CurrentStage, EnemySpawnTimer, FormationState,
    StageConfigurations,
};

pub fn spawn_enemies(
    mut commands: Commands,
    time: Res<Time>,
    mut timer: ResMut<EnemySpawnTimer>,
    mut stage: ResMut<CurrentStage>,
    mut formation_state: ResMut<FormationState>,
    stage_configs: Res<StageConfigurations>, // Use imported name
) {
    // Get current stage config, looping if stage number exceeds defined configs
    let config_index = (stage.number as usize - 1) % stage_configs.stages.len();
    let current_config = &stage_configs.stages[config_index];
    let stage_enemy_count = current_config.enemy_count;

    // Tick the timer every frame
    timer.timer.tick(time.delta());

    // Only spawn if we haven't spawned the full formation for this stage yet
    // AND the timer just finished this frame
    if formation_state.total_spawned_this_stage < stage_enemy_count
        && timer.timer.just_finished()
    {
        let slot_index = formation_state.next_slot_index;

        // Ensure slot_index is within bounds of the formation layout
        if slot_index >= current_config.formation_layout.positions.len() {
             println!("Warning: slot_index {} out of bounds for formation layout '{}' (size {})",
                slot_index, current_config.formation_layout.name, current_config.formation_layout.positions.len());
             // Optionally, reset the timer and skip spawning this frame, or handle differently
             timer.timer.reset();
             return;
        }

        // Get target position from the stage's formation layout
        let target_pos = current_config.formation_layout.positions[slot_index];

        // Spawn position (random at the top) - Corrected to use WINDOW_WIDTH
        let spawn_x = (fastrand::f32() - 0.5) * (WINDOW_WIDTH - ENEMY_SIZE.x);
        let spawn_y = WINDOW_HEIGHT / 2.0 + ENEMY_SIZE.y / 2.0; // Spawn slightly above screen

        // Determine enemy type - now with a chance to spawn Boss enemies
        // Higher stages have a slightly higher boss chance
        let boss_chance = 0.05 + (stage.number as f32 * 0.01).min(0.15);
        let enemy_type = if fastrand::f32() < boss_chance {
            println!("Spawning a Boss enemy!");
            EnemyType::Boss
        } else {
            EnemyType::Grunt
        };

        // Determine sprite color based on type
        let sprite_color = match enemy_type {
            EnemyType::Grunt => Color::rgb(1.0, 0.2, 0.2),
            EnemyType::Boss => Color::rgb(0.8, 0.2, 1.0),
        };

        commands.spawn((
            SpriteBundle {
                sprite: Sprite {
                    color: sprite_color,
                    custom_size: Some(ENEMY_SIZE),
                    ..default()
                },
                transform: Transform::from_translation(Vec3::new(spawn_x, spawn_y, 0.0)),
                ..default()
            },
            Enemy {
                enemy_type,
                // Use shoot interval from stage config
                shoot_cooldown: Timer::new(
                    Duration::from_secs_f32(current_config.enemy_shoot_interval),
                    TimerMode::Once,
                ),
            },
            FormationTarget { position: target_pos },
            EnemyState::Entering,
        ));

        // Use stage_enemy_count for cycling index
        formation_state.next_slot_index = (slot_index + 1) % stage_enemy_count;
        formation_state.total_spawned_this_stage += 1;

        // Mark that we are now waiting for enemies to be cleared
        stage.waiting_for_clear = true;

        // Reset timer only if we are still spawning more enemies for the formation
        if formation_state.total_spawned_this_stage < stage_enemy_count {
            timer.timer.reset();
        } else {
            println!(
                "Full formation ({}) spawned for Stage {}",
                current_config.formation_layout.name, stage.number
            );
        }
    }
}

pub fn move_enemies(
    mut entering_query: Query<
        (Entity, &mut Transform, &FormationTarget, &mut EnemyState),
        (With<Enemy>, With<FormationTarget>),
    >,
    mut attacking_query: Query<
        (Entity, &mut Transform, &EnemyState, &Enemy), // Add &Enemy here
        (With<Enemy>, Without<FormationTarget>),
    >, // Query potential attackers
    time: Res<Time>,
    mut commands: Commands,
    stage: Res<CurrentStage>,
    stage_configs: Res<StageConfigurations>, // Add stage configurations
    has_beam_query: Query<&TractorBeam>,
) {
    // Get current stage config for speed multiplier
    let config_index = (stage.number as usize - 1) % stage_configs.stages.len();
    let current_config = &stage_configs.stages[config_index];
    let speed_multiplier = current_config.enemy_speed_multiplier;

    // Calculate speeds for this frame
    let base_speed = ENEMY_SPEED * speed_multiplier;
    let attack_speed = base_speed * 1.5; // Attackers are faster
    let arrival_threshold = base_speed * time.delta_seconds() * 1.1; // Threshold for reaching formation target

    // --- Handle Entering Enemies ---
    for (entity, mut transform, target, mut state) in entering_query.iter_mut() {
        // Ensure we only process Entering state here, though query filters it mostly
        if *state == EnemyState::Entering {
            let current_pos = transform.translation;
            let target_pos = target.position;
            // Using target_pos which is already a Vec3, not a reference
            let direction = target_pos - current_pos;
            let distance = direction.length();

            if distance < arrival_threshold {
                // Arrived at target
                transform.translation = target_pos;
                commands.entity(entity).remove::<FormationTarget>(); // Remove target component
                *state = EnemyState::InFormation; // Change state
                println!(
                    "Enemy {:?} reached formation target and is now InFormation.",
                    entity
                );
            } else {
                // Move towards target using base_speed
                let move_delta = direction.normalize() * base_speed * time.delta_seconds();
                transform.translation += move_delta;
            }
        }
    }

    // --- Handle Attacking Enemies ---
    for (entity, mut transform, state, enemy) in attacking_query.iter_mut() {
        // Check what state the enemy is in
        if let EnemyState::Attacking(attack_pattern) = state {
            // Apply different movement based on enemy type
            match enemy.enemy_type {
                EnemyType::Grunt => {
                    // Basic enemies follow their attack pattern
                    match attack_pattern {
                        // ... existing patterns ...
                        AttackPattern::SwoopDive => {
                            // ... existing code ...
                            let vertical_movement = attack_speed * time.delta_seconds();
                            let horizontal_speed_factor = 0.5;
                            let horizontal_movement = if transform.translation.x < 0.0 {
                                attack_speed * horizontal_speed_factor * time.delta_seconds()
                            } else if transform.translation.x > 0.0 {
                                -attack_speed * horizontal_speed_factor * time.delta_seconds()
                            } else { 0.0 };

                            transform.translation.y -= vertical_movement;
                            transform.translation.x += horizontal_movement;

                            // Prevent overshooting center
                            if (transform.translation.x > 0.0 && transform.translation.x + horizontal_movement < 0.0) ||
                               (transform.translation.x < 0.0 && transform.translation.x + horizontal_movement > 0.0) {
                                transform.translation.x = 0.0;
                            }
                        }
                        AttackPattern::DirectDive => {
                            transform.translation.y -= attack_speed * time.delta_seconds();
                        }
                        AttackPattern::Kamikaze(target) => {
                            // Copy the target value rather than dereferencing
                            // since target should actually be a Vec3 in this context
                            let target_pos = *target; // Dereference here
                            let direction = target_pos - transform.translation;
                            let distance = direction.length();
                            let kamikaze_threshold = attack_speed * time.delta_seconds() * 1.1; // Threshold to stop near target

                            if distance > kamikaze_threshold {
                                let move_delta = direction.normalize() * attack_speed * time.delta_seconds();
                                transform.translation += move_delta;
                            } else {
                                // Optionally stop or continue past target - for now, just stop moving towards it
                                // Could also despawn here if desired upon reaching target
                            }
                        }
                        // New CaptureBeam pattern - Bosses behave differently
                        AttackPattern::CaptureBeam => {
                            // For Grunt enemies, just do a direct dive (fallback)
                            transform.translation.y -= attack_speed * time.delta_seconds();
                        }
                    }
                }
                EnemyType::Boss => {
                    // Boss has special behavior, especially for CaptureBeam
                    match attack_pattern {
                        AttackPattern::CaptureBeam => {
                            // Boss moves down to a position above the player area
                            let target_y = -WINDOW_HEIGHT / 4.0; // Position at lower quarter of the screen
                            
                            if transform.translation.y > target_y {
                                // Move down to position
                                transform.translation.y -= attack_speed * 0.8 * time.delta_seconds();
                            } else {
                                // Once in position, stay there briefly before activating beam
                                // Check if this boss already has a TractorBeam component
                                if has_beam_query.get(entity).is_err() {
                                    // Spawn tractor beam component on this boss
                                    commands.entity(entity).insert(TractorBeam {
                                        target: Entity::PLACEHOLDER, // Will be filled in by the boss_capture_attack
                                        timer: Timer::new(Duration::from_secs_f32(TRACTOR_BEAM_DURATION), TimerMode::Once),
                                        width: TRACTOR_BEAM_WIDTH,
                                        active: false,
                                    });
                                }
                            }
                        }
                        AttackPattern::SwoopDive => {
                            // ... existing code for swoop dive ...
                            let center_x = 0.0;
                            let bottom_y = -WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y;
            
                            // First move towards center-bottom
                            let target = Vec3::new(center_x, bottom_y, 0.0);
                            // target is directly created as Vec3, not a reference
                            let direction = target - transform.translation;
                            
                            // Normalize and move
                            if direction.length() > 0.0 {
                                let normalized_dir = direction.normalize();
                                transform.translation += normalized_dir * attack_speed * time.delta_seconds();
                            }
                        }
                        AttackPattern::DirectDive => {
                            transform.translation.y -= attack_speed * time.delta_seconds();
                        }
                        AttackPattern::Kamikaze(target) => {
                            // Convert the target to a value type
                            let target_pos = *target; // Dereference here
                            let direction = target_pos - transform.translation;
                            
                            // If very close to target, just move straight down
                            if direction.length() < 50.0 {
                                transform.translation.y -= attack_speed * time.delta_seconds();
                            } else {
                                // Move toward target
                                let normalized_dir = direction.normalize();
                                transform.translation += normalized_dir * attack_speed * time.delta_seconds();
                            }
                        }
                    }
                }
            }
        }

        // Despawn if off screen (This should be inside the loop)
        if transform.translation.y < -WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y {
            println!(
                "Despawning enemy {:?} that went off screen.", // Generic message as it could be InFormation or Attacking
                entity
            );
            commands.entity(entity).despawn();
            // TODO: Later, attacking enemies might return to formation or loop
        }
    } // Closes for loop
}

// System to check if all spawned enemies have reached their formation position
pub fn check_formation_complete(
    mut formation_state: ResMut<FormationState>,
    enemy_query: Query<&EnemyState, With<Enemy>>,
    mut attack_dive_timer: ResMut<AttackDiveTimer>,
    stage: Res<CurrentStage>, // Need current stage
    stage_configs: Res<StageConfigurations>, // Need stage configs
) {
    // Only run the check if the formation isn't already marked as complete
    if !formation_state.formation_complete {
        // Get current stage config
        let config_index = (stage.number as usize - 1) % stage_configs.stages.len();
        let current_config = &stage_configs.stages[config_index];
        let stage_enemy_count = current_config.enemy_count;

        // Check if all enemies for *this stage* have been spawned
        if formation_state.total_spawned_this_stage == stage_enemy_count {
            // Check if any enemies are still in the Entering state
            let mut any_entering = false;
            for state in enemy_query.iter() {
                // Use matches! macro for safety
                if matches!(state, EnemyState::Entering) {
                    any_entering = true;
                    break;
                }
            }

            // If all spawned and none are entering, formation is complete
            if !any_entering {
                 println!(
                    "Formation complete for Stage {}! Setting attack timer. (Spawned={})",
                    stage.number, formation_state.total_spawned_this_stage
                );
                formation_state.formation_complete = true;

                // Set timer duration based on stage config
                let dive_interval = Duration::from_secs_f32(current_config.attack_dive_interval);
                attack_dive_timer.timer.set_duration(dive_interval);
                attack_dive_timer.timer.reset();
                attack_dive_timer.timer.unpause();
                println!("Attack timer set to {:?} duration, unpaused and reset.", dive_interval);
            }
        }
    }
}

use crate::components::{AttackPattern, Player}; // Import the new enum and Player

pub fn trigger_attack_dives(
    mut timer: ResMut<AttackDiveTimer>,
    time: Res<Time>,
    mut enemy_query: Query<(Entity, &mut EnemyState, &Enemy)>, // Added Enemy component to check type
    formation_state: Res<FormationState>,
    stage: Res<CurrentStage>, // Need current stage
    stage_configs: Res<StageConfigurations>, // Need stage configs
    player_query: Query<&Transform, With<Player>>, // Need player position for Kamikaze
) {
    timer.timer.tick(time.delta());

    // Only proceed if the timer finished AND the formation is complete
    if timer.timer.just_finished() && formation_state.formation_complete {
        // Get the current stage config
        let config_index = (stage.number as usize - 1) % stage_configs.stages.len();
        let current_config = &stage_configs.stages[config_index];

        // Find all enemies currently in formation
        let mut available_enemies: Vec<(Entity, EnemyType)> = Vec::new();
        for (entity, state, enemy) in enemy_query.iter() {
            // Check the state correctly and store enemy type
            if matches!(state, EnemyState::InFormation) {
                available_enemies.push((entity, enemy.enemy_type));
            }
        }

        // If there are enemies available, pick one randomly
        if !available_enemies.is_empty() && !current_config.attack_patterns.is_empty() {
            let random_index = fastrand::usize(..available_enemies.len());
            let (chosen_entity, enemy_type) = available_enemies[random_index];

            // Select an attack pattern based on enemy type
            let mut selected_pattern = match enemy_type {
                // For Boss enemies, occasionally use the CaptureBeam pattern
                EnemyType::Boss => {
                    if fastrand::f32() < 0.4 { // 40% chance for Boss to use CaptureBeam
                        AttackPattern::CaptureBeam
                    } else {
                        // Otherwise use a random pattern from the stage config
                        let pattern_index = fastrand::usize(..current_config.attack_patterns.len());
                        current_config.attack_patterns[pattern_index]
                    }
                },
                // Regular enemies use patterns from the stage config
                EnemyType::Grunt => {
                    let pattern_index = fastrand::usize(..current_config.attack_patterns.len());
                    current_config.attack_patterns[pattern_index]
                }
            };

            // If Kamikaze, get player position (if player exists)
            if let AttackPattern::Kamikaze(_) = selected_pattern {
                if let Ok(player_transform) = player_query.get_single() {
                     selected_pattern = AttackPattern::Kamikaze(player_transform.translation);
                } else {
                    // Fallback if player doesn't exist (e.g., just died)
                    selected_pattern = AttackPattern::DirectDive; // Or SwoopDive
                    println!("Kamikaze target not found, falling back to DirectDive");
                }
            }

            // Get the chosen enemy's state mutably and change it
            if let Ok((_, mut state, _)) = enemy_query.get_mut(chosen_entity) {
                println!("Enemy {:?} starting attack dive with pattern {:?}!", chosen_entity, selected_pattern);
                *state = EnemyState::Attacking(selected_pattern); // Set state with pattern
                // Timer duration is handled elsewhere (e.g., check_formation_complete)
            }
        }
    }
}

pub fn enemy_shoot(
    mut commands: Commands,
    time: Res<Time>,
    // Query attacking enemies: need their transform and mutable Enemy component for the timer
    mut enemy_query: Query<(&Transform, &mut Enemy, &EnemyState), Without<FormationTarget>>, // Query remains the same
) {
    for (transform, mut enemy, state) in enemy_query.iter_mut() {
        // Only shoot if in any Attacking state (pattern doesn't matter for shooting)
        if matches!(state, EnemyState::Attacking(_)) { // Use matches! macro
            enemy.shoot_cooldown.tick(time.delta());
            if enemy.shoot_cooldown.finished() {
                // println!("Enemy {:?} firing!", transform.translation); // Less verbose logging
                let bullet_start_pos = transform.translation
                    - Vec3::Y * (ENEMY_SIZE.y / 2.0 + ENEMY_BULLET_SIZE.y / 2.0);

                commands.spawn((
                    SpriteBundle {
                        sprite: Sprite {
                            color: Color::rgb(1.0, 0.5, 0.5),
                            custom_size: Some(ENEMY_BULLET_SIZE),
                            ..default()
                        },
                        transform: Transform::from_translation(bullet_start_pos),
                        ..default()
                    },
                    EnemyBullet,
                ));

                // Reset the timer for the next shot
                enemy.shoot_cooldown.reset();
            }
        }
    }
}

// Check collisions between enemy bullets and the player
// Moved to player.rs as it affects player state directly

// New run condition: Check if the formation is complete
pub fn is_formation_complete(formation_state: Res<FormationState>) -> bool {
    formation_state.formation_complete
}

// New system to handle the tractor beam attack from Boss enemies
pub fn boss_capture_attack(
    mut commands: Commands,
    time: Res<Time>,
    mut boss_query: Query<(Entity, &Transform, &mut TractorBeam)>,
    player_query: Query<(Entity, &Transform), (With<Player>, Without<Captured>)>, // Only target non-captured players
    has_beam_query: Query<&TractorBeam>,
) {
    for (boss_entity, boss_transform, mut tractor_beam) in boss_query.iter_mut() {
        // Tick the beam timer
        tractor_beam.timer.tick(time.delta());
        
        // If player exists and beam is not active yet, set player as target
        if !tractor_beam.active && tractor_beam.target == Entity::PLACEHOLDER {
            if let Ok((player_entity, _)) = player_query.get_single() {
                tractor_beam.target = player_entity;
                tractor_beam.active = true;
                println!("Boss {:?} activated tractor beam targeting player!", boss_entity);
                
                // Create visual beam effect (using a simple sprite for now)
                let beam_height = boss_transform.translation.y - (-WINDOW_HEIGHT / 2.0); // Height from boss to bottom of screen
                
                // Spawn the beam as a child of the boss
                commands.entity(boss_entity).with_children(|parent| {
                    parent.spawn(SpriteBundle {
                        sprite: Sprite {
                            color: TRACTOR_BEAM_COLOR,
                            custom_size: Some(Vec2::new(tractor_beam.width, beam_height)),
                            ..default()
                        },
                        transform: Transform::from_xyz(0.0, -beam_height/2.0, 0.0),
                        ..default()
                    });
                });
            }
        }
        
        // If beam is active, check if player is in beam's path
        if tractor_beam.active {
            if let Ok((player_entity, player_transform)) = player_query.get_single() {
                // Check if player is roughly under the boss
                if (player_transform.translation.x - boss_transform.translation.x).abs() < tractor_beam.width / 2.0 {
                    // Player is in the beam! Capture them
                    println!("Player captured by boss {:?}!", boss_entity);
                    
                    // Add Captured component to player
                    commands.entity(player_entity).insert(Captured {
                        boss_entity,
                        timer: Timer::new(Duration::from_secs_f32(CAPTURE_DURATION), TimerMode::Once),
                    });
                    
                    // Boss returns to formation with captured player
                    commands.entity(boss_entity).remove::<TractorBeam>();
                    
                    // TODO: Implement logic for boss to return to formation
                    // For now, just despawn the boss to simplify
                    commands.entity(boss_entity).despawn_recursive();
                    break;
                }
            }
        }
        
        // If beam timer finishes and player wasn't captured, end the beam attack
        if tractor_beam.timer.finished() {
            println!("Boss {:?} tractor beam expired", boss_entity);
            commands.entity(boss_entity).remove::<TractorBeam>();
            
            // If we had a proper visual beam, we'd despawn it here
            // For now, just make sure we clean up by despawning all children
            commands.entity(boss_entity).despawn_descendants();
            
            // For simplicity, after beam attack the boss flies off-screen
            // In a more complete implementation, it might return to formation
        }
    }
}